@@ -23,9 +23,11 @@ int Read_Command_Line(option *io, int argc, char **argv)
   int idx;
   int i;
   int writemode;
-
+  short int opt_m;
+  
   writemode = WRITE;
-
+  opt_m = 0;
+  
   if(argc == 1) Exit("\n. No argument was passed to the program. Please check the documentation. \n");
   PhyML_Printf("",writemode);
   
@@ -128,6 +130,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
       {"leave_duplicates",    no_argument,NULL,81},
       {"precision",           required_argument,NULL,82},
       {"l_min",               required_argument,NULL,83},
+      {"print_mat_and_exit",  no_argument,NULL,84},
       {0,0,0,0}
     };
 
@@ -141,10 +144,15 @@ int Read_Command_Line(option *io, int argc, char **argv)
 
     do
     {     
-      c = getopt_long(argc,argv,"qi:d:m:b:n:t:f:zk:v:c:a:u:ho:s:x:g:l:ep",longopts,&idx);
+      c = getopt_long(argc,argv,"qi:d:t:m:b:n:f:zk:v:c:a:u:ho:s:x:g:l:ep",longopts,&idx);
 
       switch(c)
 	{
+        case 84 :
+          {
+            io->print_mat_and_exit = YES;
+            break;
+          }
         case 83 :
           {
             io->mod->l_min = String_To_Dbl(optarg);
@@ -364,15 +372,20 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	    s = (char *)mCalloc(T_MAX_NAME,sizeof(char));
 	    i = 0;
 	    while(optarg[i++]) s[i]=tolower(optarg[i]);
-	    if(!strcmp(optarg,"gbd")) io->rates->model               = THORNE;
-	    else if(!strcmp(optarg,"geom")) io->rates->model         = GUINDON;
-	    else if(!strcmp(optarg,"lognorm")) io->rates->model      = LOGNORMAL;
-	    else if(!strcmp(optarg,"clock")) io->rates->model        = STRICTCLOCK;
-	    else if(!strcmp(optarg,"strictclock")) io->rates->model  = STRICTCLOCK;
-	    else if(!strcmp(optarg,"strict_clock")) io->rates->model = STRICTCLOCK;
+	    if(!strcmp(optarg,"gbd")) io->rates->model_id = THORNE;
+	    else if(!strcmp(optarg,"autocorrelated")) io->rates->model_id = THORNE;
+	    else if(!strcmp(optarg,"autocorr")) io->rates->model_id = THORNE;
+	    else if(!strcmp(optarg,"geom")) io->rates->model_id = GUINDON;
+	    else if(!strcmp(optarg,"integrated")) io->rates->model_id = GUINDON;
+	    else if(!strcmp(optarg,"uncorrelated")) io->rates->model_id = LOGNORMAL;
+	    else if(!strcmp(optarg,"uncorr")) io->rates->model_id = LOGNORMAL;
+	    else if(!strcmp(optarg,"lognorm")) io->rates->model_id = LOGNORMAL;
+	    else if(!strcmp(optarg,"clock")) io->rates->model_id = STRICTCLOCK;
+	    else if(!strcmp(optarg,"strictclock")) io->rates->model_id = STRICTCLOCK;
+	    else if(!strcmp(optarg,"strict_clock")) io->rates->model_id = STRICTCLOCK;
 	    else 
 	      {
-		PhyML_Printf("\n. rate_model should be 'gbs', 'gbd', 'gamma' or 'clock'.");
+		PhyML_Printf("\n. rate_model should be 'autocorrelated', 'uncorrelated', 'integrated' or 'clock'.");
 		Exit("\n");
 	      }
 	    Free(s);
@@ -852,7 +865,8 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	    break;
 	  }
 	case 'm': case 5 :
-	  {            
+	  {
+            opt_m = 1;
 	    if (!isalpha(optarg[0]))
               {
                 if(strchr(optarg,',') == NULL)
@@ -1109,8 +1123,8 @@ int Read_Command_Line(option *io, int argc, char **argv)
                     io->do_alrt           = NO;
 		    io->ratio_test        = 0;
 		    io->n_boot_replicates = (int)atoi(optarg);
-		    io->print_boot_trees  = 1;
-		    
+		    io->print_boot_trees  = YES;
+
 		    if(io->n_data_sets > 1)
 		      {
 			char choix;
@@ -1299,7 +1313,13 @@ int Read_Command_Line(option *io, int argc, char **argv)
 	  
 	case 't':case 11:
 	  {
-            if ((io->mod->whichmodel != JC69) && (io->mod->whichmodel != F81) && (io->mod->whichmodel != GTR))
+            if(opt_m == 0)
+              {
+                PhyML_Fprintf(stderr,"\n. Please use the -m option before -t in the command line.");
+                Exit("\n");
+              }
+            
+            if((io->mod->whichmodel != JC69) && (io->mod->whichmodel != F81) && (io->mod->whichmodel != GTR))
               {
                 if ((strcmp(optarg, "e") == 0) ||
                     (strcmp(optarg, "E") == 0) ||
@@ -1308,15 +1328,13 @@ int Read_Command_Line(option *io, int argc, char **argv)
                   {
                     io->mod->kappa->v = 4.0;
                     io->mod->s_opt->opt_kappa = YES;
-                    if (io->mod->whichmodel == TN93)
-                      io->mod->s_opt->opt_lambda   = YES;
+                    if(io->mod->whichmodel == TN93) io->mod->s_opt->opt_lambda = YES;
                   }
                 else
                   {
                     io->mod->s_opt->opt_kappa  = NO;
                     io->mod->s_opt->opt_lambda = NO;
 
-                    
                     // Added the 2 TsTv ratios for TN93
                     // lambda is the ratio of both TsTv ratios
                     // kappa is the mean of both TsTv ratios
@@ -1330,12 +1348,16 @@ int Read_Command_Line(option *io, int argc, char **argv)
                           }
                         else
                           {
-                            PhyML_Fprintf(stderr,"\n. The TN93 model requires two Ts/Tv ratios.\n");
+                            PhyML_Fprintf(stderr,"\n. The TN93 model requires two ts/tv ratios.\n");
                             Exit("\n");
                           }
                         if ( (TsTvPur < .0) || (TsTvPyr < .0) )
                           {
-                            PhyML_Fprintf(stderr,"\n. The Ts/Tv ratio must be a positive number.\n");
+                            PhyML_Fprintf(stderr,"\n. ts/tv for purines: %f",TsTvPur);
+                            PhyML_Fprintf(stderr,"\n. ts/tv for pyrimidines: %f",TsTvPyr);
+                            PhyML_Fprintf(stderr,"\n. The TN93 model requires two ts/tv ratios");
+                            PhyML_Fprintf(stderr,"\n. The command-line option should look as follows: ... -t 3.5,4.3 ... ");
+                            PhyML_Fprintf(stderr,"\n. The ts/tv ratio must be a positive number.\n");
                             Exit("\n");
                           }
                         io->mod->lambda->v = (phydbl)(TsTvPur / TsTvPyr);
@@ -1347,7 +1369,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
                         if (atof(optarg) < .0)
                           {
                             char choix;
-                            PhyML_Printf("\n. The Ts/Tv ratio must be a positive number\n");
+                            PhyML_Printf("\n. The ts/tv ratio must be a positive number\n");
                             PhyML_Printf("\n. Type any key to exit.\n");
                             if(!scanf("%c",&choix)) Exit("\n");
                             Exit("\n");
@@ -1630,7 +1652,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
       io->fp_out_trees = Openfile(io->out_trees_file,1);
     }
   
-  if((io->print_boot_trees) && (io->do_boot == YES) && (io->fp_in_align != NULL))
+  if((io->print_boot_trees) && (io->do_boot == YES || io->do_tbe == YES) && (io->fp_in_align != NULL))
     {
       strcpy(io->out_boot_tree_file,io->in_align_file);
       strcat(io->out_boot_tree_file,"_phyml_boot_trees");
@@ -1638,6 +1660,7 @@ int Read_Command_Line(option *io, int argc, char **argv)
       strcat(io->out_boot_tree_file,".txt");
       io->fp_out_boot_tree = Openfile(io->out_boot_tree_file,1);
       
+      
       strcpy(io->out_boot_stats_file,io->in_align_file);
       strcat(io->out_boot_stats_file,"_phyml_boot_stats");
       if(io->append_run_ID) { strcat(io->out_boot_stats_file,"_"); strcat(io->out_boot_stats_file,io->run_id_string); }
