@@ -33,8 +33,6 @@ the GNU public licence. See http://www.opensource.org for details.
 #include "beagle_utils.h"
 #endif
 
-
-
 #if (defined PHYML || EVOLVE)
 
 int main(int argc, char **argv)
@@ -141,16 +139,30 @@ int main(int argc, char **argv)
                     if(!io->quiet) PhyML_Printf("\n\n. [Random start %3d/%3d]",num_rand_tree+1,io->mod->s_opt->n_rand_starts);
 
                   Init_Model(cdata,mod,io);
-
+                  Set_Model_Parameters(mod);
+                  
 #ifdef M4
                   if(io->mod->use_m4mod) M4_Init_Model(mod->m4mod,cdata,mod);
 #endif
-
+                  
                   // Make the initial tree
                   switch(io->in_tree)
                     {
-                    case 0 : case 1 : { tree = Dist_And_BioNJ(cdata,mod,io); break; }
-                    case 2 :          { tree = Read_User_Tree(cdata,mod,io); break; }
+                    case 0 : case 1 :
+                      {
+                        tree = Dist_And_BioNJ(cdata,mod,io);
+                        if(io->print_mat_and_exit == YES)
+                          {
+                            Print_Mat(ML_Dist(cdata,mod));
+                            exit(-1);
+                          }
+                        break;
+                      }
+                    case 2 :
+                      {
+                        tree = Read_User_Tree(cdata,mod,io);
+                        break;
+                      }
                     }
 
                   if(io->mod->s_opt->opt_topo == YES) Remove_Duplicates(cdata,io,tree);
@@ -232,13 +244,11 @@ int main(int argc, char **argv)
 #ifdef PHYML
                   if(tree->io->print_json_trace == YES) JSON_Tree_Io(tree,tree->io->fp_out_json_trace); 
 
-
                   
                   Set_Update_Eigen(YES,tree->mod);                 
                   Lk(NULL,tree);
                   Set_Update_Eigen(NO,tree->mod);
                   
-                  
                   if(tree->mod->s_opt->opt_topo)
 		    {
                       Global_Spr_Search(tree);                      
@@ -312,7 +322,7 @@ int main(int argc, char **argv)
 #ifdef BEAGLE
                   finalize_beagle_instance(tree);
 #endif
-                  Free_One_Spr(tree->best_spr);
+                  Free_Best_Spr(tree);
                   Free_Spr_List_One_Edge(tree);
                   Free_Spr_List_All_Edge(tree);
                   Free_Tree_Pars(tree);
@@ -592,16 +602,22 @@ int main(int argc, char **argv)
   for(i=0;i<io->n_otu;i++)
     {
       sscanf(io->data[i]->name,"%d",&year);
-      PhyML_Printf("\n<clade id=\"clad%d\">",i+1);
-      PhyML_Printf("\n\t<taxon value=\"%s\"/>",io->data[i]->name);
-      PhyML_Printf("\n</clade>");
-      PhyML_Printf("\n<calibration id=\"cal%d\">",i+1);
-      PhyML_Printf("\n\t<lower>%d</lower>",0);
-      PhyML_Printf("\n\t<upper>%d</upper>",0);
-      PhyML_Printf("\n\t<appliesto clade.id=\"clad%d\"/>",i+1);
-      PhyML_Printf("\n</calibration>");
+      PhyML_Printf("\n%s\t%d",io->data[i]->name,year);
     }
 
+  /* for(i=0;i<io->n_otu;i++) */
+  /*   { */
+  /*     sscanf(io->data[i]->name,"%d",&year); */
+  /*     PhyML_Printf("\n<clade id=\"clad%d\">",i+1); */
+  /*     PhyML_Printf("\n\t<taxon value=\"%s\"/>",io->data[i]->name); */
+  /*     PhyML_Printf("\n</clade>"); */
+  /*     PhyML_Printf("\n<calibration id=\"cal%d\">",i+1); */
+  /*     PhyML_Printf("\n\t<lower>%d</lower>",year); */
+  /*     PhyML_Printf("\n\t<upper>%d</upper>",year); */
+  /*     PhyML_Printf("\n\t<appliesto clade.id=\"clad%d\"/>",i+1); */
+  /*     PhyML_Printf("\n</calibration>"); */
+  /*   } */
+
   /* FILE *fp; */
   /* char *name,*date; */
   /* int i; */
