@@ -704,6 +704,7 @@ int Update_RAS(t_mod *mod)
             mod->ras->gamma_r_proba_unscaled->v[mod->ras->n_catg-1];
         }
 
+#ifndef PHYREX
       do
         {
           sum = .0;
@@ -716,10 +717,10 @@ int Update_RAS(t_mod *mod)
           for(i=0;i<mod->ras->n_catg;i++) mod->ras->gamma_r_proba->v[i]/=sum;
         }
       while((sum > 1.01) || (sum < 0.99));
-
+#endif
+      
 
       // Update class rates
-
       if(mod->ras->normalise_rr == YES)
         {
           sum = .0;
@@ -779,7 +780,7 @@ int Update_Efrq(t_mod *mod)
 //////////////////////////////////////////////////////////////
 
 int Set_Model_Parameters(t_mod *mod)
-{
+{  
   if(!Update_Boundaries(mod)) return 0;
   if(!Update_RAS(mod))        return 0;
   if(!Update_Efrq(mod))       return 0;
@@ -828,6 +829,7 @@ int Update_Boundaries(t_mod *mod)
       for(i=0;i<6;i++) if(mod->r_mat->rr->v[i] > RR_MAX) mod->r_mat->rr->v[i] = RR_MAX;
     }
 
+
   for(i=0;i<mod->ns;i++)
     {
       if(mod->e_frq->pi_unscaled->v[i] < UNSCALED_E_FRQ_MIN)
@@ -835,7 +837,7 @@ int Update_Boundaries(t_mod *mod)
 
       if(mod->e_frq->pi_unscaled->v[i] > UNSCALED_E_FRQ_MAX)
         mod->e_frq->pi_unscaled->v[i] = UNSCALED_E_FRQ_MAX;
-            
+
       if(mod->e_frq->pi->v[i] < E_FRQ_MIN)
         mod->e_frq->pi->v[i] = E_FRQ_MIN;
 
