@@ -661,18 +661,6 @@ phydbl TIMES_Lk_Yule_Order(t_tree *tree)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-phydbl TIMES_Lk_Times(int verbose, t_tree *tree)
-{
-  DATE_Assign_Primary_Calibration(tree);
-  DATE_Update_T_Prior_MinMax(tree);
-  tree->times->c_lnL_times =  TIMES_Lk_Birth_Death(verbose,tree);
-  return(tree->times->c_lnL_times);
-}
-
-//////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////
-
-
 void TIMES_Lk_Times_Trav(t_node *a, t_node *d, phydbl lim_inf, phydbl lim_sup, phydbl *logdens, t_tree *tree)
 {
   int i;
@@ -865,45 +853,322 @@ phydbl TIMES_Wrap_Lk_Coalescent(t_edge *b, t_tree *tree, supert_tree *stree)
 ////////////////////////////////////////////////////////////*/
 
 phydbl TIMES_Lk_Coalescent(t_tree *tree)
-{
+{  
   t_node *n;
   int n_lineages;
-  phydbl lnP;
+  phydbl lnP,Ne,dt,exp_g;
+  phydbl T;
+  
+  Ne = tree->times->scaled_pop_size;
+  exp_g = tree->times->neff_growth;
+  
+  if(Ne > tree->times->scaled_pop_size_max || Ne < tree->times->scaled_pop_size_min)
+    {
+      tree->times->c_lnL = UNLIKELY;
+      return(tree->times->c_lnL);
+    }
   
   Get_Node_Ranks_From_Times(tree);
 
-
+  n = tree->n_root;
+  while(n->rk_next) n = n->rk_next;
+  T = tree->times->nd_t[n->num];
+  
   lnP = 0.0;
   n_lineages = 1;
   n = tree->n_root;
   while(n->rk_next)
     {
+      dt = fabs(tree->times->nd_t[n->num] - tree->times->nd_t[n->rk_next->num]);
+      
+      assert(!(tree->times->nd_t[n->num] > tree->times->nd_t[n->rk_next->num]));
+      
       if(n->tax == YES) n_lineages--;
       else n_lineages++;
+
+
+      if(tree->times->coalescent_model_id == EXPCOALESCENT)
+        {
+          dt =
+            1./exp_g *
+            (exp(exp_g * fabs(tree->times->nd_t[n->num]-T)) -
+             exp(exp_g * fabs(tree->times->nd_t[n->rk_next->num]-T)));
+        }
+      else if(tree->times->coalescent_model_id == POWLAW)
+        {
+          dt =
+            1./(1.+exp_g) *
+            (pow(fabs(tree->times->nd_t[n->num]-T)+1,exp_g+1.) -
+             pow(fabs(tree->times->nd_t[n->rk_next->num]-T)+1,exp_g+1.));
+        }
       
-      lnP += -n_lineages * (n_lineages-1.) / (2.*tree->times->scaled_pop_size) * fabs(tree->times->nd_t[n->num] - tree->times->nd_t[n->rk_next->num]);
+      lnP += -n_lineages * (n_lineages-1.) / (2.*Ne) * dt;
       
+      if(tree->times->coalescent_model_id == EXPCOALESCENT && n->tax == NO)
+        lnP += exp_g * fabs(tree->times->nd_t[n->num]-T);
+      else if(tree->times->coalescent_model_id == POWLAW && n->tax == NO)
+        lnP += exp_g * log(fabs(tree->times->nd_t[n->num]-T) + 1.);
+      
+      if(n->tax == NO) lnP -= log(Ne);
+      
+      // Multifurcations not allowed under standard Kingman coalescent
+      if(fabs(tree->times->nd_t[n->num] - tree->times->nd_t[n->rk_next->num]) < SMALL &&
+         n->tax == NO && n->rk_next->tax == NO)
+        {
+          tree->times->c_lnL = UNLIKELY;
+          return(UNLIKELY);
+        }
+            
+      if((isnan(lnP) || isinf(lnP)) == TRUE)
+        {
+          tree->times->c_lnL = UNLIKELY;
+          return(UNLIKELY);
+        }
+        
       n = n->rk_next;
     }
+  
+  tree->times->c_lnL = lnP;
 
-  lnP -= (tree->n_otu-1) * log(tree->times->scaled_pop_size);
-  
-  tree->times->c_lnL_times = lnP;
-  
   return(lnP);
-  
 }
 
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-  phydbl TIMES_Lk_Uniform_Core(t_tree *tree)
+phydbl TIMES_Lk_Coalescent_Range(t_dsk *young, t_dsk *old, t_tree *tree)
+{
+  int n_lineages;
+  phydbl disk_lnP,lnP,Ne,dt,exp_g,T;
+  t_dsk *disk;
+
+  Ne = tree->times->scaled_pop_size;  
+  exp_g = tree->times->neff_growth;
+  T = tree->young_disk->time;
+  
+  if(Ne > tree->times->scaled_pop_size_max || Ne < tree->times->scaled_pop_size_min) return(UNLIKELY);
+
+  
+  
+  lnP = 0.0;
+  disk_lnP = 0.0;
+  disk = young->prev;
+  do
+    {
+      n_lineages = disk->next->n_ldsk_a;
+      n_lineages -= (disk->next->ldsk ? (disk->next->ldsk->n_next -1) : 0);
+
+      dt = 0.0;
+      
+      if(tree->times->coalescent_model_id == STRICTCOALESCENT)
+        dt = fabs(disk->time - disk->next->time);
+      else if(tree->times->coalescent_model_id == EXPCOALESCENT)
+        dt =
+          1./exp_g *
+          (exp(exp_g * fabs(disk->time-T)) -
+           exp(exp_g * fabs(disk->next->time-T)));
+
+
+      disk_lnP = -n_lineages * (n_lineages-1.) / (2.*Ne) * dt;
+
+      if(tree->times->coalescent_model_id == EXPCOALESCENT)
+        disk_lnP += exp_g * fabs(disk->time-T);
+      
+      lnP += disk_lnP;
+
+      // Multifurcations not allowed under standard Kingman coalescent
+      if(fabs(disk->time - disk->next->time) < SMALL &&
+         disk->ldsk && disk->ldsk->n_next > 1 &&
+         disk->next->ldsk && disk->next->ldsk->n_next > 1)
+        {
+          tree->times->c_lnL = UNLIKELY;
+          return(UNLIKELY);
+        }
+
+      assert((isnan(lnP) || isinf(lnP)) == FALSE);
+      disk = disk->prev;
+    }
+  while(disk != NULL && disk != old->prev);
+  
+  lnP -= (tree->n_otu-1) * log(Ne);
+
+  return(lnP);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+phydbl TIMES_Prior_Coalescent(t_tree *tree)
+{
+  phydbl lbda;
+  lbda = 1./tree->times->neff_prior_mean;
+  return(log(lbda)-lbda*tree->times->scaled_pop_size);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+phydbl TIMES_Prior(t_tree *tree)
+{
+  tree->times->c_lnP = UNLIKELY;
+    
+  switch(tree->times->model_id)
+    {
+    case COALESCENT :
+      {
+        tree->times->c_lnP = TIMES_Prior_Coalescent(tree);
+        break;
+      }
+    case SLFV_GAUSSIAN : case SLFV_UNIFORM :
+      {
+        tree->times->c_lnP = TIMES_Prior_SLFV(tree);
+        break;
+      }
+    case BIRTHDEATH : 
+      {
+        tree->times->c_lnP = UNLIKELY;
+        break;
+      }     
+    default : break;
+    }
+
+  return(tree->times->c_lnP);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+void TIMES_Simulate_Coalescent(t_tree *tree)
+{
+  t_node *n;
+  phydbl t,dt,Ne,*times,u,coal_rate,prob_coal;
+  int n_lineages,*permut,available_idx,i;
+  t_ll *lineages;
+  t_node *v1, *v2, *a;
+  
+  
+  Ne = tree->times->scaled_pop_size;
+  times = tree->times->nd_t;
+  lineages = NULL;
+  
+  Get_Node_Ranks_From_Tip_Times(tree);
+
+
+  // Start from the most recent tip (or set of tips)
+  n = tree->a_nodes[tree->n_otu-1];  
+  while(n->rk_next) n = n->rk_next;
+  
+  n_lineages = 0;
+  available_idx = tree->n_otu;
+  dt = 0.0;
+  t = tree->times->nd_t[n->num];
+  do
+    {
+      if(n->tax == YES)
+        {
+          Push_Bottom_Linked_List(n,&lineages,NO);
+          n_lineages++;
+        }
+      
+      coal_rate = n_lineages * (n_lineages - 1.) / (2.*Ne);
+
+      // Proba next coalescent event is in [T[n->num],T[n->rk_prev->num] 
+      prob_coal = 1. - exp(-coal_rate * fabs(times[n->num] - (n->rk_prev ? times[n->rk_prev->num] : -INFINITY)));
+            
+      u = Uni();
+      
+      if(!(u > prob_coal))
+        {
+          dt = Rexp_Trunc(coal_rate,
+                          0.0,
+                          times[n->num] - (n->rk_prev ? times[n->rk_prev->num] : -INFINITY));
+                          
+          t = t - dt;
+                    
+          permut = Permutate(n_lineages);
+
+          a = tree->a_nodes[available_idx];
+          v1 = Linked_List_Elem(permut[0],lineages);
+          v2 = Linked_List_Elem(permut[1],lineages);
+          available_idx++;
+
+          a->v[0]  = NULL;
+          a->v[1]  = v1;
+          a->v[2]  = v2;
+          v1->v[0] = a;
+          v2->v[0] = a;
+
+          times[a->num] = t;
+          
+          Remove_From_Linked_List(NULL,v1,&lineages);
+          Remove_From_Linked_List(NULL,v2,&lineages);
+          Push_Bottom_Linked_List(a,&lineages,NO);
+
+          a->rk_prev = n->rk_prev;
+          n->rk_prev = a;
+          a->rk_next = n;
+          
+          n_lineages--;
+        }
+      
+      n = n->rk_prev;
+      t = tree->times->nd_t[n->num];
+    }
+  while(!(n_lineages == 1 && n->rk_prev == NULL));
+
+  for(i=0;i<3;++i) 
+    if(tree->n_root->v[1]->v[i] == tree->n_root) 
+      { 
+        tree->n_root->v[1]->v[i] = tree->n_root->v[2]; 
+        break; 
+      }
+  
+  for(i=0;i<3;++i) 
+    if(tree->n_root->v[2]->v[i] == tree->n_root) 
+      { 
+        tree->n_root->v[2]->v[i] = tree->n_root->v[1]; 
+        break; 
+      }
+  
+  Connect_Edges_To_Nodes_Serial(tree);
+  
+  tree->e_root = NULL;
+  for(i=0;i<2*tree->n_otu-3;++i)
+    {
+      if((tree->a_edges[i]->left == tree->n_root->v[1] && tree->a_edges[i]->rght == tree->n_root->v[2]) ||
+         (tree->a_edges[i]->left == tree->n_root->v[2] && tree->a_edges[i]->rght == tree->n_root->v[1]))
+        {        
+          tree->e_root = tree->a_edges[i];
+          break;
+        }
+    }
+  assert(!(tree->e_root == NULL));
+  
+  tree->n_root->b[1] = tree->a_edges[2*tree->n_otu-3];
+  tree->n_root->b[2] = tree->a_edges[2*tree->n_otu-2];
+  
+  tree->n_root->b[1]->left = tree->n_root;
+  tree->n_root->b[1]->rght = tree->n_root->v[1];
+  
+  tree->n_root->b[2]->left = tree->n_root;
+  tree->n_root->b[2]->rght = tree->n_root->v[2];
+  
+  Update_Ancestors(tree->n_root,tree->n_root->v[2],tree->n_root->b[2],tree);
+  Update_Ancestors(tree->n_root,tree->n_root->v[1],tree->n_root->b[1],tree);
+  
+  MIXT_Propagate_Tree_Update(tree);  
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+phydbl TIMES_Lk_Uniform_Core(t_tree *tree)
 {  
   phydbl logn;
 
   logn = TIMES_Log_Number_Of_Ranked_Labelled_Histories(tree->n_root,YES,tree);
 
-  tree->times->c_lnL_times = 0.0;
+  tree->times->c_lnL = 0.0;
   TIMES_Lk_Uniform_Post(tree->n_root,tree->n_root->v[2],tree);
   TIMES_Lk_Uniform_Post(tree->n_root,tree->n_root->v[1],tree);
 
@@ -915,15 +1180,15 @@ phydbl TIMES_Lk_Coalescent(t_tree *tree)
   /* 	 log(tree->times->time_slice_lims[tree->times->curr_slice[tree->n_root->num]] - */
   /* 	     tree->times->nd_t[tree->n_root->num])); */
 
-  tree->times->c_lnL_times +=
+  tree->times->c_lnL +=
     Factln(tree->rates->n_tips_below[tree->n_root->num]-2.) -
     (phydbl)(tree->rates->n_tips_below[tree->n_root->num]-2.) *
     log(tree->times->time_slice_lims[tree->times->curr_slice[tree->n_root->num]] -
   	tree->times->nd_t[tree->n_root->num]);
   
-  tree->times->c_lnL_times -= logn;
+  tree->times->c_lnL -= logn;
   
-  return(tree->times->c_lnL_times);
+  return(tree->times->c_lnL);
 }
 
 //////////////////////////////////////////////////////////////
@@ -1024,7 +1289,7 @@ void TIMES_Lk_Uniform_Post(t_node *a, t_node *d, t_tree *tree)
       
       if(tree->times->curr_slice[a->num] != tree->times->curr_slice[d->num])
 	{
-	  tree->times->c_lnL_times += 
+	  tree->times->c_lnL += 
 	    Factln(tree->rates->n_tips_below[d->num]-1.) - 
 	    (phydbl)(tree->rates->n_tips_below[d->num]-1.) *
 	    log(tree->times->time_slice_lims[tree->times->curr_slice[d->num]] -
@@ -1306,7 +1571,7 @@ phydbl TIMES_Lk_Birth_Death(int verbose, t_tree *tree)
   
   if(b < d)
     {
-      tree->times->c_lnL_times = UNLIKELY;
+      tree->times->c_lnL = UNLIKELY;
       if(verbose) PhyML_Printf("\n. b < d");
       return UNLIKELY;
     }
@@ -1318,7 +1583,7 @@ phydbl TIMES_Lk_Birth_Death(int verbose, t_tree *tree)
         if(tree->times->nd_t[i] < tree->times->t_prior_min[i] ||
            tree->times->nd_t[i] > tree->times->t_prior_max[i]) 
           {
-            tree->times->c_lnL_times = UNLIKELY;
+            tree->times->c_lnL = UNLIKELY;
             if(verbose)
               {
                 PhyML_Printf("\n. Time outside calibration range : %G [%G,%G]",tree->times->nd_t[i],tree->times->t_prior_min[i],tree->times->t_prior_max[i]);
@@ -1365,7 +1630,7 @@ phydbl TIMES_Lk_Birth_Death(int verbose, t_tree *tree)
             if(!(lnL > UNLIKELY))
               {
                 PhyML_Printf("\n. logb: %G pt: %G nut1: %G lognut1: %G t: %G logp_1t: %G\n",logb,pt,nut1,lognut1,t,logp_1t);
-                tree->times->c_lnL_times = UNLIKELY;
+                tree->times->c_lnL = UNLIKELY;
                 return UNLIKELY;
               }
           }
@@ -1403,7 +1668,7 @@ phydbl TIMES_Lk_Birth_Death(int verbose, t_tree *tree)
             if(!(lnL > UNLIKELY))              
               {
                 PhyML_Printf("\n. lognut1: %G t: %G logp_1t: %G",lognut1,t,logp_1t);
-                tree->times->c_lnL_times = UNLIKELY;
+                tree->times->c_lnL = UNLIKELY;
                 return UNLIKELY;
               }
           }
@@ -1426,7 +1691,7 @@ phydbl TIMES_Lk_Birth_Death(int verbose, t_tree *tree)
   else if(b < bmin && d > dmin) 
     {
       PhyML_Printf("\n. b: %G bmin: %G d: %G dmin: %G",b,bmin,d,dmin);
-      tree->times->c_lnL_times = UNLIKELY;
+      tree->times->c_lnL = UNLIKELY;
       return UNLIKELY;
     }
   else if(Are_Equal(bmd,0.0,bmin/10.) == YES) // Critical birth-death process
@@ -1448,7 +1713,7 @@ phydbl TIMES_Lk_Birth_Death(int verbose, t_tree *tree)
             if(!(lnL > UNLIKELY))              
               {
                 PhyML_Printf("\n. logb: %G t: %G",logb,t);
-                tree->times->c_lnL_times = UNLIKELY;
+                tree->times->c_lnL = UNLIKELY;
                 return UNLIKELY;
               }
           }
@@ -1470,13 +1735,13 @@ phydbl TIMES_Lk_Birth_Death(int verbose, t_tree *tree)
   else if(b < bmin && d < dmin) // Birth and death rates are below their limits
     {
       PhyML_Fprintf(stderr,"\n. b: %G bmin: %G d: %G dmin: %G",b,bmin,d,dmin);
-      tree->times->c_lnL_times = UNLIKELY;
+      tree->times->c_lnL = UNLIKELY;
       return -INFINITY;
     }
   else if(b > bmax && d > dmax)
     {
       PhyML_Fprintf(stderr,"\n. b: %G bmax: %G d: %G dmax: %G",b,bmax,d,dmax);
-      tree->times->c_lnL_times = UNLIKELY;
+      tree->times->c_lnL = UNLIKELY;
       return -INFINITY;
     }
   else
@@ -1487,11 +1752,11 @@ phydbl TIMES_Lk_Birth_Death(int verbose, t_tree *tree)
   if(isnan(lnL) || isinf(fabs(lnL)) || !(lnL > UNLIKELY))
     {
       PhyML_Fprintf(stderr,"\n. lnL times: %f",lnL);
-      tree->times->c_lnL_times = UNLIKELY;
+      tree->times->c_lnL = UNLIKELY;
       return UNLIKELY;
     }
 
-  tree->times->c_lnL_times = lnL;
+  tree->times->c_lnL = lnL;
 
   return(lnL);
 }
@@ -1871,8 +2136,8 @@ void TIMES_Randomize_Tree_With_Time_Constraints(t_cal *cal_list, t_tree *mixt_tr
             }
         }
       
-      Update_Ancestors(mixt_tree->n_root,mixt_tree->n_root->v[2],mixt_tree);
-      Update_Ancestors(mixt_tree->n_root,mixt_tree->n_root->v[1],mixt_tree);
+      Update_Ancestors(mixt_tree->n_root,mixt_tree->n_root->v[2],mixt_tree->n_root->b[2],mixt_tree);
+      Update_Ancestors(mixt_tree->n_root,mixt_tree->n_root->v[1],mixt_tree->n_root->b[1],mixt_tree);
       DATE_Assign_Primary_Calibration(mixt_tree);
       DATE_Update_T_Prior_MinMax(mixt_tree);
       
@@ -2167,9 +2432,6 @@ phydbl TIMES_Tree_Length(t_tree *tree)
       sum += fabs(tree->times->nd_t[tree->a_edges[i]->left->num]-
                   tree->times->nd_t[tree->a_edges[i]->rght->num]);
 
-  sum += fabs(tree->times->nd_t[tree->n_root->num] - tree->times->nd_t[tree->n_root->v[1]->num]);
-  sum += fabs(tree->times->nd_t[tree->n_root->num] - tree->times->nd_t[tree->n_root->v[2]->num]);
-
   return(sum);
 }
 
@@ -2270,12 +2532,29 @@ void TIMES_Copy_Time_Struct(t_time *from, t_time *to, int n_otu)
   to->death_rate_max = from->death_rate_max;
   to->death_rate_pivot = from->death_rate_pivot;
   
-  to->c_lnL_times = from->c_lnL_times;
+  to->c_lnL = from->c_lnL;
   to->c_lnL_jps = from->c_lnL_jps;
   
   to->nd_t_recorded = from->nd_t_recorded;
   to->update_time_norm_const = from->update_time_norm_const;
 
+  to->scaled_pop_size = from->scaled_pop_size;
+  to->scaled_pop_size_min = from->scaled_pop_size_min;
+  to->scaled_pop_size_max = from->scaled_pop_size_max;
+
+  to->neff_growth = from->neff_growth;
+  to->neff_growth_min = from->neff_growth_min;
+  to->neff_growth_max = from->neff_growth_max;
+
+  to->model_id = from->model_id;
+  to->coalescent_model_id = from->coalescent_model_id;
+
+  to->update_time_norm_const = from->update_time_norm_const;
+
+  to->augmented_coalescent = from->augmented_coalescent;
+
+  to->neff_prior_mean = from->neff_prior_mean;
+  
   for(i=0;i<2*n_otu-1;++i) to->nd_t[i] = from->nd_t[i];
   for(i=0;i<2*n_otu-1;++i) to->buff_t[i] = from->buff_t[i];
   for(i=0;i<2*n_otu-1;++i) to->true_t[i] = from->true_t[i];
@@ -2303,15 +2582,135 @@ void TIMES_Copy_Time_Struct(t_time *from, t_time *to, int n_otu)
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
-//////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////
+phydbl TIMES_Lk(t_tree *tree)
+{
+  switch(tree->times->model_id)
+    {
+    case COALESCENT :
+      {
+        tree->times->c_lnL = TIMES_Lk_Coalescent(tree);
+        break;
+      }
+    case SLFV_GAUSSIAN : case SLFV_UNIFORM :
+      {
+        tree->times->c_lnL = TIMES_Lk_SLFV(tree);        
+        break;
+      }
+    case BIRTHDEATH : 
+      {
+        DATE_Assign_Primary_Calibration(tree);
+        DATE_Update_T_Prior_MinMax(tree);
+        tree->times->c_lnL =  TIMES_Lk_Birth_Death(NO,tree);
+        break;
+      }     
+    default : break;
+    }
+
+  return(tree->times->c_lnL);
+}
 
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+phydbl TIMES_Lk_Range(t_dsk *young, t_dsk *old, t_tree *tree)
+{
+  switch(tree->times->model_id)
+    {
+    case COALESCENT :
+      {
+        return(TIMES_Lk_Coalescent_Range(young,old,tree));
+        break;
+      }
+    case SLFV_GAUSSIAN : case SLFV_UNIFORM :
+      {
+        return(TIMES_Lk_SLFV_Range(young,old,tree));
+        break;
+      }
+    default : break;
+    }
+  return(-1.);
+}
+
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
+phydbl TIMES_Prior_SLFV(t_tree *tree)
+{
+  return(UNLIKELY);
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+phydbl TIMES_Lk_SLFV(t_tree *tree)
+{
+  phydbl lnL;
+  t_dsk *disk;
+  phydbl dt;
+  int n_evt;
+
+  n_evt = 0;
+  dt    = 0.0;
+  lnL   = 0.0;
+  disk  = tree->young_disk->prev;
+  do
+    {
+      if(disk->age_fixed == NO)
+        {
+          dt += fabs(disk->next->time - disk->time);
+          n_evt++;
+        }
+      
+      disk = disk->prev;
+    }
+  while(disk);
+
+  lnL += n_evt * log(tree->mmod->lbda) - tree->mmod->lbda * dt;
+
+  tree->times->c_lnL = lnL;
+  
+  return(tree->times->c_lnL);  
+}
+
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+
+phydbl TIMES_Lk_SLFV_Range(t_dsk *young, t_dsk *old, t_tree *tree)
+{
+  t_dsk *disk;
+  int n_evt;
+  phydbl lnP,dt;
+
+  assert(young);
+
+  lnP   = 0.0;    
+  dt    = 0.0;
+  n_evt = 0;
+  disk = young->prev;
+  do
+    {
+      if(disk->age_fixed == NO)
+        {
+          dt += fabs(disk->next->time - disk->time);
+          n_evt++;
+        }
+
+      assert(disk);
+      if(disk->time > disk->next->time) return UNLIKELY;
+      if(disk == old) break;
+      disk = disk->prev;
+    }
+  while(disk);
+
+  lnP = n_evt * log(tree->mmod->lbda) - tree->mmod->lbda * dt;
+ 
+  return(lnP);
+}
+  
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 
