@@ -34,19 +34,19 @@ the GNU public licence. See http://www.opensource.org for details.
 /* #include <malloc/malloc.h> */
 /* #include <malloc.h> */
 
-#if (defined(__AVX__))
+#if (defined(__AVX__) || defined(__AVX2__))
 #include <xmmintrin.h>
 #include <pmmintrin.h>
 #include <immintrin.h>
-#elif (defined(__SSE3__))
+#elif (defined(__SSE__) || defined(__SSE2__) || defined(__SSE3__))
 #include <emmintrin.h>
 #include <pmmintrin.h>
 #endif
 
 
-#if !defined(__FMA__) && defined(__AVX2__)
-    #define __FMA__ 1
-#endif
+/* #if !defined(__FMA__) && (defined(__AVX__) || defined(__AVX2__)) */
+/* #define __FMA__ 1 */
+/* #endif */
 
 extern int n_sec1;
 extern int n_sec2;
@@ -94,10 +94,18 @@ static inline int isinf_d  (double      x) { return isnan (x - x); }
 static inline int isinf_ld (long double x) { return isnan (x - x); }
 #endif
 
-#define SLFV_GAUSSIAN 0 /* Spatial Lambda-Fleming-Viot model (Gaussian) */
-#define SLFV_UNIFORM 1 /* Spatial Lambda-Fleming-Viot model (Uniform) */
-#define RW  2 /* standard Brownian diffusion model in phylogeography */
-#define RRW  3 /* Lemey's relaxed random walk */
+#define BIRTHDEATH 0
+#define COALESCENT 1
+
+#define STRICTCOALESCENT 0
+#define EXPCOALESCENT 1
+#define POWLAW 2
+
+#define SLFV_GAUSSIAN 2 /* Spatial Lambda-Fleming-Viot model (Gaussian) */
+#define SLFV_UNIFORM 3 /* Spatial Lambda-Fleming-Viot model (Uniform) */
+#define RW  4 /* standard Brownian diffusion model in phylogeography */
+#define RRW_GAMMA  5 /* Lemey's relaxed random walk (Gamma distributed relative diffusion rates) */
+#define RRW_LOGNORMAL  6 /* Lemey's relaxed random walk (Lognormal distributed relative diffusion rates) */
 
 #define AC 0
 #define AG 1
@@ -106,18 +114,23 @@ static inline int isinf_ld (long double x) { return isnan (x - x); }
 #define CT 4
 #define GT 5
 
-#if (defined __AVX__)
+#if (defined __AVX__ || defined __AVX2__)
 #define BYTE_ALIGN 32
-#elif (defined __SSE3__)
+#elif (defined(__SSE__) || defined(__SSE2__) ||  defined __SSE3__)
 #define BYTE_ALIGN 16
 #else
 #define BYTE_ALIGN 1
 #endif
 
 #ifndef M_1_SQRT_2PI
-#define M_1_SQRT_2PI	0.398942280401432677939946059934	/* 1/sqrt(2pi) */
+#define M_1_SQRT_2PI	0.398942280401432677939946059934	/* 1/sqrt(2*pi) */
 #endif
 
+#ifndef LOG_SQRT_2PI
+#define LOG_SQRT_2PI	0.91893853320467266954	/* log(sqrt(2*pi)) */
+#endif
+
+
 // verbose levels
 #define VL0 0
 #define VL1 1
@@ -147,7 +160,7 @@ static inline int isinf_ld (long double x) { return isnan (x - x); }
 
 
 #define  TSTV_MIN 0.05
-#define  TSTV_MAX 20.0
+#define  TSTV_MAX 100.0
 
 #define  PINV_MIN 0.00001
 #define  PINV_MAX 0.99999
@@ -230,6 +243,10 @@ static inline int isinf_ld (long double x) { return isnan (x - x); }
 #define  FALSE 0
 #endif
 
+#define SPATIAL_SAMPLING_DETECTION 0
+#define SPATIAL_SAMPLING_SURVEY 1
+
+
 #define  ON  1
 #define  OFF 0
 
@@ -337,7 +354,6 @@ static inline int isinf_ld (long double x) { return isnan (x - x); }
 // Amino acid ordering:
 // Ala Arg Asn Asp Cys Gln Glu Gly His Ile Leu Lys Met Phe Pro Ser Thr Trp Tyr Val
 
-#define COALESCENT 0
 
 #define COMPOUND_COR   0
 #define COMPOUND_NOCOR 1
@@ -346,7 +362,6 @@ static inline int isinf_ld (long double x) { return isnan (x - x); }
 #define THORNE         4
 #define GUINDON        5
 #define STRICTCLOCK    6
-#define BIRTHDEATH     7
 #define NONE          -1
 
 #define ALRTSTAT       1
@@ -526,6 +541,7 @@ typedef struct __Node {
   struct __Node               ***bip_node; /*! three lists of pointer to tip nodes. One list for each direction */
   struct __Edge                       **b; /*! table of pointers to neighbor branches */
   struct __Node                      *anc; /*! direct ancestor t_node (for rooted tree only) */
+  struct __Edge                    *b_anc; /*! edge between this node and its direct ancestor (for rooted tree only) */
   struct __Node                 *ext_node;
   struct __Node               *match_node;
   struct __Align                   *c_seq; /*! corresponding compressed sequence */
@@ -629,11 +645,9 @@ typedef struct __Edge {
   short int           *div_post_pred_rght; /*! posterior prediction of nucleotide/aa diversity (rght-hand subtree) */
   short int                    does_exist;
 
-
-
   phydbl            *p_lk_left,*p_lk_rght; /*! likelihoods of the subtree on the left and right side (for each site and each relative rate category) */
   phydbl         *p_lk_tip_r, *p_lk_tip_l;
-
+  
 
 #ifdef BEAGLE
   int        p_lk_left_idx, p_lk_rght_idx;
@@ -716,7 +730,7 @@ typedef struct __Tree{
   struct __Tree                    *next_mixt; /*! set to NULL by default. Used for mixture models */
   struct __Tree                    *prev_mixt; /*! set to NULL by default. Used for mixture models */
   struct __Tree                    *mixt_tree; /*! set to NULL by default. Used for mixture models */
-  struct __Tree                   *extra_tree; /*! set to NULL by default. Used a latent variable in molecular dating */
+  struct __Tree                    **aux_tree; /*! set to NULL by default. Used as a latent variable in molecular dating */
   struct __Option                         *io; /*! input/output */
   struct __Matrix                        *mat; /*! pairwise distance matrix */
   struct __Node                   **curr_path; /*! list of nodes that form a path in the tree */
@@ -735,8 +749,17 @@ typedef struct __Tree{
   struct __Generic_LL              *edge_list;
   struct __Generic_LL              *node_list;
   struct __Independent_Contrasts       *ctrst; /*! Pointer to data structure used for independent contrasts */
-  
 
+#if (defined(__AVX__) || defined(__AVX2__))
+  __m256d *_tPij1,*_tPij2,*_pmat1plk1,*_pmat2plk2,*_plk0,*_l_ev,*_r_ev,*_prod_left,*_prod_rght;
+#elif (defined(__SSE__) || defined(__SSE2__) || defined(__SSE3__))
+  __m128d *_tPij1,*_tPij2,*_pmat1plk1,*_pmat2plk2,*_plk0,*_l_ev,*_r_ev,*_prod_left,*_prod_rght;
+#endif
+
+  phydbl                  *p_lk_left_pi,*l_ev;
+  phydbl                        *big_lk_array;
+  int                        big_lk_array_pos;
+  
   short int                         eval_alnL; /*! Evaluate likelihood for genetic data */
   short int                         eval_rlnL; /*! Evaluate likelihood for rates along the tree */
   short int                         eval_glnL; /*! Evaluate tree likelihood */
@@ -768,14 +791,12 @@ typedef struct __Tree{
   int                                      dp; /*! Data partition */
   int                               s_mod_num; /*! Substitution model number */
   int                               lock_topo; /*! = 1 any subsequent topological modification will be banished */
-  int                            write_labels;
+  int                            print_labels;
   int                           write_br_lens;
   int                                 *mutmap; /*! Mutational map */
   int                                json_num;
   short int                   update_eigen_lr;
   int                                tip_root; /*! Index of tip node used as the root */
-  phydbl                       *eigen_lr_left;
-  phydbl                       *eigen_lr_rght;
   phydbl                            *dot_prod;
 
   phydbl                                *expl;
@@ -784,6 +805,7 @@ typedef struct __Tree{
   phydbl                             best_lnL; /*! highest value of the loglikelihood found so far */
   int                               best_pars; /*! highest value of the parsimony found so far */
   phydbl                                c_lnL; /*! loglikelihood */
+  phydbl                                p_lnL; /*! loglikelihood (previous value) */
   phydbl                              old_lnL; /*! old loglikelihood */
   phydbl                    sum_min_sum_scale; /*! common factor of scaling factors */
   phydbl                        *c_lnL_sorted; /*! used to compute c_lnL by adding sorted terms to minimize CPU errors */
@@ -1313,7 +1335,8 @@ typedef struct __Option { /*! mostly used in 'help.c' */
   int                leave_duplicates;/* Leave duplicated sequences */
   int                       precision;/* Decimal output precision for values in stats file */
   int               n_boot_replicates;
-  
+
+  short int        print_mat_and_exit;
   short int            print_node_num; /*! print node numbers if print_node_num=1 */
   short int         print_support_val;
   
@@ -1523,8 +1546,17 @@ typedef struct __T_Rate {
   phydbl less_likely;
   phydbl c_lnL1;
   phydbl c_lnL2;
-  phydbl c_lnL_rates; /*! Prob(Br len | time stamps, model of rate evolution) */
+
+  phydbl c_lnL; /*! Current value of ProbDens(Br len | time stamps, model of rate evolution) */
+  phydbl p_lnL; /*! Previous value of c_lnL */
+
+  phydbl c_lnP; /*! Current value of ProbDens(time stamps, model of rate evolution) */
+  phydbl p_lnP; /*! Previous value of c_lnP */
+
   phydbl clock_r; /*! Mean substitution rate, i.e., 'molecular clock' rate */
+  phydbl clock_r_prior_mean;
+  phydbl clock_r_prior_var;
+  short int clock_r_has_prior;
   phydbl min_clock;
   phydbl max_clock;
   phydbl lbda_nu;
@@ -1578,7 +1610,7 @@ typedef struct __T_Rate {
   int use_rates; /*! if = 0, branch lengths are expressed as differences between t_node times */
   int bl_from_rt; /*! if =1, branch lengths are obtained as the product of cur_r and t */
   int approx;
-  int model; /*! Model number */
+  int model_id; /*! Model number */
   char *model_name;
   int is_allocated;
   int met_within_gibbs;
@@ -1611,11 +1643,21 @@ typedef struct __T_Time {
   phydbl *buff_t; /*! Current t_node times */
   phydbl *true_t; /*! Current t_node times */
 
-  phydbl c_lnL_times; /*! Prob(time stamps) */
+  phydbl c_lnL; /*! ProbDens(time stamps|param) */
+  phydbl c_lnP; /*! ProbDens(param) */
+  phydbl p_lnL; /*! Previous value of Prob(time stamps|param) */
+  phydbl p_lnP; /*! Previous value of Prob(param) */
+
   phydbl c_lnL_jps; /*! Prob(# Jumps | time stamps, rates, model of rate evolution) */
 
   phydbl scaled_pop_size; // Product of effective population size with length of a generation in calendar time unit
+  phydbl scaled_pop_size_min; 
+  phydbl scaled_pop_size_max; 
 
+  phydbl neff_growth;
+  phydbl neff_growth_min;
+  phydbl neff_growth_max;
+  
   phydbl birth_rate;
   phydbl birth_rate_min;
   phydbl birth_rate_max;
@@ -1662,9 +1704,15 @@ typedef struct __T_Time {
 
   phydbl *mean_t; /*! average values of nd_t taken across the sampled values during the MCMC */
 
-  int model;
-
+  short int model_id;
+  short int coalescent_model_id;
+  
   int update_time_norm_const;
+
+  short int augmented_coalescent;
+
+  phydbl neff_prior_mean;
+  
 }t_time;
 
 /*!********************************************************/
@@ -1674,6 +1722,7 @@ typedef struct __Tmcmc {
 
   phydbl *tune_move;
   phydbl *move_weight;
+  phydbl *move_prob;
 
   phydbl *acc_rate;
   int *acc_move;
@@ -1684,6 +1733,9 @@ typedef struct __Tmcmc {
   int *move_type;
   char **move_name;
 
+  time_t time_beg;
+  time_t time_end;
+  
   int num_move_nd_r;
   int num_move_br_r;
   int num_move_times;
@@ -1728,11 +1780,14 @@ typedef struct __Tmcmc {
   int num_move_phyrex_swap_disk;
   int num_move_phyrex_indel_hit;
   int num_move_phyrex_spr;
-  int num_move_phyrex_spr_local;
+  int num_move_phyrex_spr_slide;
+  int num_move_phyrex_narrow_exchange;
+  int num_move_phyrex_wide_exchange;
   int num_move_phyrex_scale_times;
   int num_move_phyrex_ldscape_lim;
   int num_move_phyrex_sigsq;
-  int num_move_phyrex_neff;
+  int num_move_time_neff;
+  int num_move_time_neff_growth;
   int num_move_phyrex_sim;
   int num_move_phyrex_traj;
   int num_move_phyrex_indel_disk_serial;
@@ -1747,6 +1802,8 @@ typedef struct __Tmcmc {
   int num_move_clade_change;
   int num_move_phyrex_ldsk_tip_to_root;
   int num_move_phyrex_sigsq_scale;
+  int num_move_phyrex_ldsk_tips;
+  int num_move_phyrex_node_times;
 
   int nd_t_digits;
   int *monitor;
@@ -1921,7 +1978,11 @@ typedef struct __Phylogeo{
   phydbl           min_lbda;
   phydbl           max_lbda;
 
-  phydbl              c_lnL;
+  phydbl              c_lnL; // ProbDens(location|param) 
+  phydbl              p_lnL; // Previous value of c_lnL
+
+  phydbl              c_lnP; // ProbDens(param)
+  phydbl              p_lnP; // Previous value of p_lnP
 
   struct __Node **sorted_nd; // Table of nodes sorted wrt their heights.
 
@@ -1936,7 +1997,7 @@ typedef struct __Phylogeo{
 }t_geo;
 
 /*!********************************************************/
-// Structure for the Etheridge-Barton migration/reproduction model
+// Data structure for the migration/reproduction model
 typedef struct __Migrep_Model{
   struct __Geo_Coord         *lim_up; // max longitude and lattitude                        
   struct __Geo_Coord         *lim_do; // min longitude and lattitude                        
@@ -1944,8 +2005,10 @@ typedef struct __Migrep_Model{
 
   phydbl             sigsq_scale_min;
   phydbl             sigsq_scale_max;
+
+  phydbl               rrw_norm_fact;
   
-  short int                       id;
+  short int                 model_id;
   int                          n_dim;
   int                    safe_phyrex;
   int           max_num_of_intervals;
@@ -1966,7 +2029,7 @@ typedef struct __Migrep_Model{
   phydbl                     max_rad; // max of radius of the migrep disk 
   phydbl             prior_param_rad; // parameter of the parameter for the prior on radius
 
-  phydbl                       sigsq; // parent to offspring distance variance (i.e., gene flow) parameter. 
+  phydbl                      *sigsq; // parent to offspring distance variance (i.e., gene flow) parameter. First elem is for latitude, second is for longitude 
   phydbl                   min_sigsq; // min 
   phydbl                   max_sigsq; // max  
   phydbl           prior_param_sigsq; // parameter of the parameter for the prior 
@@ -1976,6 +2039,12 @@ typedef struct __Migrep_Model{
   phydbl                          nu; // parameter of hyperprior on sigsq_scale (see Eq. (1) in Lemey et al., 2010).
   
   phydbl                       c_lnL; // current value of log-likelihood 
+  phydbl                       p_lnL; // previous value of log-likelihood 
+
+  phydbl                       c_lnP; // current value of log-prior
+  phydbl                       p_lnP; // previous value of log-prior 
+
+
   phydbl              c_ln_prior_rad; // current value of log prior for the prior on radius
   phydbl             c_ln_prior_lbda; // current value of log prior for the prior on lbda
   phydbl               c_ln_prior_mu; // current value of log prior for the prior on mu
@@ -1983,6 +2052,10 @@ typedef struct __Migrep_Model{
 
   phydbl             soft_bound_area;
 
+  short int          sampling_scheme;
+  short int            use_locations;
+
+  phydbl             disp_prior_mean;
 
 }t_phyrex_mod;
 
@@ -1990,17 +2063,23 @@ typedef struct __Migrep_Model{
 
 typedef struct __Disk_Event{
   struct __Geo_Coord      *centr;
-  phydbl                    time;
   struct __Disk_Event      *next;
   struct __Disk_Event      *prev;
+  struct __Disk_Event       *img;
   struct __Lindisk_Node **ldsk_a; // array of lindisk nodes corresponding to this disk event.
-  int                   n_ldsk_a; // size of ldsk_a.
   struct __Lindisk_Node    *ldsk;
   struct __Migrep_Model    *mmod;
+  
+  phydbl                cum_glnL; // cumulative log-likelihood (locations)
+  phydbl                cum_tlnL; // cumulative log-likelihood (times);
+  phydbl                    time;
+  phydbl                time_bkp;
+  
+  short int            age_fixed; // time is fixed for disks corresponding to samples.
+
+  int                   n_ldsk_a; // size of ldsk_a.
   char                       *id;
 
-  phydbl                   c_lnL;
-  short int            age_fixed; // time is fixed for disks corresponding to samples.
 }t_dsk;
 
 /*!********************************************************/
@@ -2026,13 +2105,21 @@ typedef struct __Geo_Coord{
 
 typedef struct __Lindisk_Node{
   struct __Disk_Event     *disk;
+  struct __Disk_Event *baseline;
   struct __Lindisk_Node  **next;
   struct __Lindisk_Node   *prev;
+  struct __Lindisk_Node    *img;
   struct __Geo_Coord     *coord; 
   struct __Geo_Coord *cpy_coord; 
-  short int              is_hit;
-  int                    n_next;
   struct __Node             *nd;
+
+  short int              is_hit;
+
+  int                    n_next;
+
+  phydbl                     rr;
+  phydbl                  sigsq;
+  phydbl              base_line;
 }t_ldsk;
 
 /*!********************************************************/
@@ -2213,7 +2300,7 @@ void Warn_And_Exit(const char *s);
 void Randomize_Sequence_Order(calign *cdata);
 void Update_Root_Pos(t_tree *tree);
 void Add_Root(t_edge *target,t_tree *tree);
-void Update_Ancestors(t_node *a,t_node *d,t_tree *tree);
+void Update_Ancestors(t_node *a, t_node *d, t_edge *b, t_tree *tree);
 #if (defined PHYTIME || defined SERGEII)
 t_tree *Generate_Random_Tree_From_Scratch(int n_otu,int rooted);
 #endif
@@ -2384,9 +2471,17 @@ t_tree *Duplicate_Tree(t_tree *ori);
 matrix *K80_dist(calign *data, phydbl g_shape);
 matrix *JC69_Dist(calign *data, t_mod *mod);
 matrix *Hamming_Dist(calign *data, t_mod *mod);
-phydbl Haversine_Distance(phydbl lon1, phydbl lat1, phydbl lon2, phydbl lat2);
 phydbl Tree_Length(t_tree *tree);
 void Remove_Duplicates_From_Tree(calign *data, t_tree *tree);
+void Reset_Lk(t_tree *tree);
+void Set_Lk(t_tree *tree);
+void Reset_Prior(t_tree *tree);
+void Set_Prior(t_tree *tree);
+t_edge *Get_Edge(t_node *a, t_node *d, t_tree *tree);
+void Exchange_Nodes(t_node *a, t_node *d, t_node *w, t_node *v, t_tree *tree);
+void Init_T_Beg(t_tree *tree);
+void Set_Ignore_Root(int yesno, t_tree *tree);
+void Set_Bl_From_Rt(int yesno, t_tree *tree);
 
 
 #include "xml.h"
@@ -2418,6 +2513,7 @@ void Remove_Duplicates_From_Tree(calign *data, t_tree *tree);
 #include "rw.h"
 #include "rrw.h"
 #include "slfv.h"
+#include "location.h"
 #endif
 
 #ifdef MPI
@@ -2438,4 +2534,10 @@ void Remove_Duplicates_From_Tree(calign *data, t_tree *tree);
 #endif
 
 
+#if (defined(__AVX__) || defined(__AVX2__))
+#include "avx.h"
+#elif (defined(__SSE__) || defined(__SSE2__) || defined(__SSE3__))
+#include "sse.h"
+#endif
+
 #endif
